"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeRule = exports.buildRules = void 0;
const fast_glob_1 = __importDefault(require("fast-glob"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const resolve_1 = require("../utils/resolve");
const constants_1 = require("../constants");
const color_func_1 = require("../color-func");
const utils_1 = require("../utils");
async function buildRules(libraryItem, series, force) {
    let findSeriesRules = await findRules(series);
    const { includes = [] } = libraryItem;
    const entry = (0, resolve_1.resolveLibrary)(libraryItem);
    if (!entry) {
        return [];
    }
    const seriesRules = [];
    includes.forEach((item) => {
        const realRule = findSeriesRules
            .map((fcr) => ({
            ...fcr,
            styleFilePath: `${item}/${fcr.styleFilePath}`,
        }))
            .filter((val) => {
            return fs_extra_1.default.existsSync(path_1.default.join(entry, val.styleFilePath));
        });
        seriesRules.push(...realRule);
    });
    if (!includes.length) {
        return seriesRules;
    }
    // 搜索组件库内所有的less
    const getPattern = (isForce) => includes.map((item) => `**/${item}/**/*${isForce ? constants_1.BACKUP_EXT : ''}.less`);
    if (force) {
        const backupFiles = fast_glob_1.default.sync(getPattern(true), {
            cwd: entry,
            absolute: true,
            ignore: ['**/vc-dialog/**'],
        });
        await Promise.all(backupFiles.map(async (backupFile) => {
            if (fs_extra_1.default.existsSync(backupFile)) {
                fs_extra_1.default.copyFileSync(backupFile, backupFile.replace(constants_1.BACKUP_EXT, ''));
            }
        }));
    }
    const files = fast_glob_1.default.sync(getPattern(false), {
        cwd: entry,
        absolute: true,
        ignore: [`**/*${constants_1.BACKUP_EXT}.less`, '**/vc-dialog/**'],
    });
    let rules = [];
    Promise.all(files.map((file) => walkFile(file, entry, rules)));
    const styleFiles = seriesRules.map((item) => item.styleFilePath);
    rules = rules.filter((rule) => {
        return !styleFiles.includes(rule.styleFilePath);
    });
    return [...rules, ...seriesRules];
}
exports.buildRules = buildRules;
async function walkFile(file, entry, rules) {
    const content = fs_extra_1.default.readFileSync(file, { encoding: 'utf-8' });
    const rule = {
        styleFilePath: path_1.default.relative(entry, file),
    };
    let change = false;
    const processColorFunc = (0, color_func_1.createProcessColorFunc)();
    for (const processFunc of processColorFunc) {
        // postcss不支持 @functions 解析
        if (content.includes(processFunc.funcName + '(') &&
            !content.includes('@functions')) {
            mergeRule(rule, processFunc);
            change = true;
        }
    }
    if (change) {
        rules.push(rule);
    }
}
// 查询自定义的rules
const findRules = async (series) => {
    const files = fast_glob_1.default.sync([`${series}/index.{ts,js}`], {
        cwd: __dirname,
        absolute: true,
    }) || [];
    const rules = [];
    for (let i = 0; i < files.length; i += 1) {
        if (/\.d\.ts$/.test(files[i])) {
            continue;
        }
        const importRet = await Promise.resolve().then(() => __importStar(require(files[i].replace(/^\./, __dirname)))).then((r) => {
            return r.default?.default ?? r.default;
        });
        rules.push(...(Array.isArray(importRet) ? importRet : [importRet]));
    }
    return rules;
};
function mergeRule(rule, processFunc) {
    const { decls, atRules, formatResult } = processFunc;
    rule.decls = [...(rule.decls || []), ...decls];
    rule.atRules = [...(rule.atRules || []), ...atRules];
    rule.formatResult = rule.formatResult
        ? (0, utils_1.compose)(rule.formatResult, formatResult)
        : formatResult;
    return rule;
}
exports.mergeRule = mergeRule;
